/* 
 * Copyright (c) 2004 Roman Bogorodskiy (bogorodskiy@inbox.ru)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * $Header: /cvsroot/phptags/phptags/src/conf.c,v 1.2 2004/07/21 11:00:21 mirrorbox Exp $
 *  
 */

#include <stdio.h>

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "conf.h"
#include "phptags.h"

int
read_config()
{
	FILE *config_file;
	char *m = (char *)malloc(255);
	char *line;
	char *local_conf   = (char *)expand_filename(PHPTAGS_LOCAL_CONF),
	     *default_conf = (char *)expand_filename(PHPTAGS_DEFAULT_CONF);
	
	line = m;
	config_file = fopen(local_conf, "r");

	if (!config_file) {
#ifdef DEBUG
		perror(local_conf);
#endif /* DEBUG */
		config_file = fopen(default_conf, "r");
		
		if (!config_file) { 
#ifdef DEBUG
			perror(default_conf);
#endif /* DEBUG */
			return -1;
		} else {
			current_conf = default_conf;
		}
	} else {
		current_conf = local_conf;
	}

	while (fgets(line, 255, config_file) || feof(config_file) == 0) {
		while (isspace(*line))
			line++;
	
		if (line[0] != '#' && strchr(line, '=') != NULL) {
			/* verbose */
			if (!strncmp(line, "verbose", 7)) {
				line = strchr(line, '=');
				line++;
				
				if (atoi(line) == 1 || atoi(line) == 0) {
					verbose = atoi(line);
#ifdef DEBUG
					printf("verbose = %i\n", verbose);
#endif /* DEBUG */
				} else
					(void)fprintf(stderr, "wrong value\n");
			}

			/* tag classes */
			if (!strncmp(line, "classes", 7)) {
				line = strchr(line, '=');
				line++;

				if (atoi(line) == 1 || atoi(line) == 0) {
					tag_classes = atoi(line);
#ifdef DEBUG
					printf("classes = %i\n", tag_classes);
#endif /* DEBUG */
				} else
					(void)fprintf(stderr, "wrong value\n");
			}

			/* tagsfile */
			if (!strncmp(line, "tagsfile", 8)) {
				line = strchr(line, '=');
				line++;
				while (isspace(*line))
					line++;

				tagsfile = (char *)malloc(255);
				sscanf(line, "%s", tagsfile);
#ifdef DEBUG
				printf("tagsfile = %s\n", tagsfile);
#endif /* DEBUG */
			}
		}
	}
	
	return 0;
}


