/*
 * Copyright (c) 2004 Roman Bogorodskiy (bogorodskiy@inbox.ru)
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the 
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in 
 * all copies or substantial portions of the Software. 
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL 
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 * DEALINGS IN THE SOFTWARE.
 *
 * $Header: /cvsroot/phptags/phptags/src/phptags.c,v 1.3 2004/11/05 15:13:43 mirrorbox Exp $
 * 
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif /* HAVE_CONFIG_H */

#include <stdio.h> 
#include <stdlib.h> 
#include <string.h> 
#include <ctype.h>
#include <unistd.h>
#include <sysexits.h>

#include "conf.h"
#include "options.h"
#include "phptags.h"
#include "str.h"

int 
main(int argc, char **argv)
{
	int i,
	    ch;

	FILE *tmp_file;
	(void)read_config();
	    
#ifdef HAVE_GETOPT_LONG
	static struct option long_options[] =
	{
		
		{ "classes", no_argument, NULL, 'c' },
		{ "defenitions", no_argument, NULL, 'd' },
		{ "tagsfile", required_argument, NULL, 'f' },
		{ "help", no_argument, NULL, 'h' },
		{ "verbose", no_argument, NULL, 'v' },
		{ "version", no_argument, NULL, 'V' },
		{ NULL, 0, NULL, 0 } /* sentinel */
	};
	
	while ((ch = getopt_long(argc, argv, "cdf:hvV", long_options, (int *)0)) != -1) {
#else
	while ((ch = getopt(argc, argv, "cdf:hvV")) != -1) {
#endif /* HAVE_GETOPT_LONG */
	switch (ch) {
			case 'c':
				tag_classes = 1;
				break;
			case 'd':
				tag_defs = 1;
			        break;
			case 'f':
				tagsfile = optarg;
			        break;
			case 'v':
				verbose = 1;
				break;
			case 'V':
			        show_version();
			        break;
			case 'h':
				help();
				break;
			default:
				usage();
		}
	}

	argc -= optind;
	argv += optind;

	if (argc == 0) 
                usage();
      
	if (verbose && (current_conf != "none")) 
		printf("Using config file: %s\n", current_conf);
		
	tmp_file = fopen(tagsfile, "w+");
	
	if ((int)tmp_file == NULL) {
		printf("Error opening file: %s\n", tagsfile);
		/* TODO give some details here */
		return -1;
	}

	if (verbose)
		printf("Output: %s\n", tagsfile);
	
	for (i = 0; i < argc; i++) {
		char *m = malloc(256);
		char *line;
		FILE *fd;
	        line = m;
		
		fd = fopen(argv[i], "r");
		if ((int)fd == NULL) {
			printf("Error opening file: %s\n", argv[i]);
			return -1;
		}
		
		if (verbose)
			printf("Analyzing %s\n", argv[i]);

		while (fgets(line, 255, fd)) {
			int commented = 0;
			
			while (isspace(*line)) 
				line++;  
			
			if (strncmp("//", line, 2) && strncmp("#", line, 1)) {
				if (strnstr(line, "/*",  255)) 
					commented = 1;
				
				if (strnstr(line, "*/",  255)) 
					commented = 0;
				
				if (!commented) {
					if (!strncmp("function ", line, 9)) {
						/* foo bar.c /^function foo($bar)$/; */
						int func_name_len;
						char *func_name;
						line+=9;
						line[strlen(line) - 1] = '\0';
						
						func_name_len = strlen(line) - strlen(strnstr(line, "(", 255));
						func_name = malloc(func_name_len);
												
						(void)memmove(func_name, line, func_name_len);
						
						/* dunno why have to do it, but it's odd */
						func_name[func_name_len] = '\0'; 
						
						(void)fprintf(tmp_file, "%s\t%s\t/^function %s$/\n", 
								func_name, argv[i], line);
					
						free(func_name);
					} else if (!strncmp("class ", line, 6) && tag_classes == 1) {
						int class_name_len;
						char *class_name;
						line+=6;
						line[strlen(line) - 1] = '\0';

						class_name_len = strlen(line) - strlen(strnstr(line, " ", 255));
						class_name = malloc(class_name_len);

						(void)strncpy(class_name, line, class_name_len);
						
						(void)fprintf(tmp_file, "%s\t%s\t/^class %s$/\n",
							      class_name, argv[i], line);
						free(class_name);
							      
					} else if (!strncmp("define(", line, 7) && tag_defs == 1) {
						/* XXX 
						 * we need to keep in mind that is's also possible to
						 * use "define ("
						 */
						int def_name_len;
						char *def_name;
						/* TODO do something */
					}
				} /* if (!commented) */
			} 
		
		} /* while (fgets(line, 255, fd)) */
		free(m);
		fclose(fd);
	}

	fclose(tmp_file);
	
	if (!fork()) {
		setsid();
		execlp("sort", "sort", "-o", tagsfile, tagsfile, NULL);
		
		exit(EX_OK);
	}

	if (verbose)
		printf("Done!\n");

	return 0;
}

static void 
show_version()
{
	
	printf("%s (c) Roman Bogorodskiy\n",  PACKAGE_STRING);

	exit(EX_OK);
}

static void 
help()
{
        
	(void)fprintf(stderr, "Usage: %s [-Vcvh] [-f tagsfile] file ...\n\n",  PACKAGE_NAME);
#ifdef HAVE_GETOPT_LONG
        (void)fprintf(stderr, "\t-f, --tagsfile  <file>         - tags filename (default: tags)\n");
	(void)fprintf(stderr, "\t-c, --classes		       - create tags for classes (default: off)\n");
	(void)fprintf(stderr, "\t-v, --verbose                  - be verbose\n");
	(void)fprintf(stderr, "\t-h, --help                     - display this text\n");
	(void)fprintf(stderr, "\t-V, --version                  - show version\n");
	(void)fprintf(stderr, "\n");
						
#else 
	(void)fprintf(stderr, "\t-f <file>               - tags filename (default: tags)\n");
	(void)fprintf(stderr, "\t-c 			- create tags for classes (default: off)\n");
	(void)fprintf(stderr, "\t-v			- be verbose\n");
	(void)fprintf(stderr, "\t-h			- display this text\n");
	(void)fprintf(stderr, "\t-V			- show version\n");
	(void)fprintf(stderr, "\n");
#endif /* HAVE_GETOPT_LONG */	
	exit(EX_OK);
}

static void
usage()
{

	(void)fprintf(stderr, "Usage: %s [-Vcvh] [ -f tagsfile ] file ...\n", PACKAGE_NAME);

	exit(EX_USAGE);
}
